 /*
 * Copyright (C) 2016 Akash Vibhute <akash.roboticist@gmail.com>
 * 
 * Library to handle MAG3110 magnetometer IMU on Virgo 3 robot. 
 * Based off various user libraries
 *
 *
 * Initial Release: Aug/01/2016
 *
 */

#ifndef MAG3110_H
#define MAG3110_H

#include "mbed.h"
//#include "generalFunctions.h"

#define MAG_ADDR (0x0E << 1)

// define registers
#define MAG_DR_STATUS 0x00
#define MAG_OUT_X_MSB 0x01
#define MAG_OUT_X_LSB 0x02
#define MAG_OUT_Y_MSB 0x03
#define MAG_OUT_Y_LSB 0x04
#define MAG_OUT_Z_MSB 0x05
#define MAG_OUT_Z_LSB 0x06
#define MAG_WHO_AM_I  0x07
#define MAG_SYSMOD    0x08
#define MAG_OFF_X_MSB 0x09
#define MAG_OFF_X_LSB 0x0A
#define MAG_OFF_Y_MSB 0x0B
#define MAG_OFF_Y_LSB 0x0C
#define MAG_OFF_Z_MSB 0x0D
#define MAG_OFF_Z_LSB 0x0E
#define MAG_DIE_TEMP  0x0F
#define MAG_CTRL_REG1 0x10
#define MAG_CTRL_REG2 0x11

// what should WHO_AM_I return?
#define MAG_3110_WHO_AM_I_VALUE 0xC4


// Fields in registers
// CTRL_REG1: dr2,dr1,dr0  os1,os0  fr tm ac

// Sampling rate from 80Hz down to 0.625Hz
#define MAG_3110_SAMPLE80 0
#define MAG_3110_SAMPLE40 0x20
#define MAG_3110_SAMPLE20 0x40
#define MAG_3110_SAMPLE10 0x60
#define MAG_3110_SAMPLE5 0x80
#define MAG_3110_SAMPLE2_5 0xA0
#define MAG_3110_SAMPLE1_25 0xC0
#define MAG_3110_SAMPLE0_625 0xE0

// How many samples to average (lowers data rate)
#define MAG_3110_OVERSAMPLE1 0
#define MAG_3110_OVERSAMPLE2 0x08
#define MAG_3110_OVERSAMPLE3 0x10
#define MAG_3110_OVERSAMPLE4 0x18

// read only 1 byte per axis
#define MAG_3110_FASTREAD 0x04
// do one measurement (even if in standby mode)
#define MAG_3110_TRIGGER 0x02
// put in active mode
#define MAG_3110_ACTIVE 0x01

// CTRL_REG2: AUTO_MRST_EN  _ RAW MAG_RST _ _ _ _ _
// reset sensor after each reading
#define MAG_3110_AUTO_MRST_EN 0x80
// don't subtract user offsets
#define MAG_3110_RAW 0x20
// reset magnetic sensor after too-large field
#define MAG_3110_MAG_RST 0x10

// DR_STATUS Register ZYXOW ZOW YOW XOW ZYXDR ZDR YDR XDR
#define MAG_3110_ZYXDR  0x08

/**
 * MAG3110 Class to read X/Y/Z data from the magentometer
 *
 */
class MAG3110
{
public:
    /**
     * Main constructor
     * @param sda SDA pin
     * @param sdl SCL pin
     */
    MAG3110(PinName sda, PinName scl);
    /**
     * Setup the Magnetometer
     * @param off_x magnetometer offset for X-Axis
     * @param off_y magnetometer offset for Y-Axis
     * @param off_z magnetometer offset for Z-Axis
     *
     */
    void begin(int16_t off_x, int16_t off_y, int16_t off_z);
    /**
     * Read a register, return its value as int
     * @param regAddr The address to read
     * @return value in register
     */
    int readReg(char regAddr);
    /**
     * Read a value from a pair of registers, return as int
     * @param regAddr The address to read
     * @return Value from 2 consecutive registers
     */
    int readVal(char regAddr);
    /**
     * Perform a read on the X, Y and Z values.
     * @param xVal Pointer to X value
     * @param yVal Pointer to Y value
     * @param zVal Pointer to Z value
     */
    void getValues(int *xVal, int *yVal, int *zVal);
    /**
     * Set the calibration parameters if required.
     * @param minX Minimum value for X range
     * @param maxX Maximum value for X range
     * @param minY Minimum value for Y range
     * @param maxY maximum value for Y range
     * @param minZ Minimum value for Z range
     * @param maxZ maximum value for Z range
     */
    void setCalibration(float minX, float maxX, float minY, float maxY, float minZ, float maxZ);
    /**
     * Acquire data from all axes in floating and return in uT, floating point format.
     * @param uT Pointer to 1x3 floating point array to store micro tesla values from X,Y,Z axes
     */
    void get_uT(float * uT);

private:
    I2C _i2c;
    float _avgX, _avgY, _avgZ;

};
#endif
